@@ -1,9 +1,11 @@
 package AI::ConfusionMatrix;
-$AI::ConfusionMatrix::VERSION = '0.002';
+$AI::ConfusionMatrix::VERSION = '0.003';
 use Carp;
 use Exporter 'import';
 our @EXPORT= qw (makeConfusionMatrix);
+use strict;
 use Tie::File;
+use warnings;
 
 # ABSTRACT: Make a confusion matrix
 
@@ -23,6 +25,8 @@ sub makeConfusionMatrix {
         $stats{$expected}{'tp'} = 0;
 
         for my $predicted (keys %{$matrix->{$expected}}) {
+            # Ensure that the False Positive counter is defined to be able to compute the total later
+            $stats{$predicted}{'fp'} //= 0;
             $stats{$expected}{'total'} += $matrix->{$expected}->{$predicted};
             $stats{$expected}{'tp'} += $matrix->{$expected}->{$predicted} if $expected == $predicted;
             if ($expected != $predicted) {
@@ -30,10 +34,11 @@ sub makeConfusionMatrix {
                 $stats{$predicted}{'fp'} += $matrix->{$expected}->{$predicted};
             }
             $totals{$predicted} += $matrix->{$expected}->{$predicted};
+            # Add the label to the array of columns if it does not contain it already
             push @columns, $predicted unless _findIndex($predicted, \@columns);
         }
 
-        $stats{$expected}{'acc'} = sprintf("%.2f%", ($stats{$expected}{'tp'} * 100) / $stats{$expected}{'total'});
+        $stats{$expected}{'acc'} = sprintf("%.2f", ($stats{$expected}{'tp'} * 100) / $stats{$expected}{'total'});
     }
 
     for my $expected (@expected) {
@@ -42,21 +47,25 @@ sub makeConfusionMatrix {
         $totals{'fn'}    += $stats{$expected}{'fn'};
         $totals{'fp'}    += $stats{$expected}{'fp'};
         $totals{'acc'}   += $stats{$expected}{'acc'};
+        $stats{$expected}{'acc'} .= '%';
     }
 
-    $totals{'acc'} = sprintf("%.2f%", $totals{'acc'} / @expected);
+    $totals{'acc'} = sprintf("%.2f%%", $totals{'acc'} / @expected);
     @columns = sort @columns;
     map {$array[0] .= ',' . $_} join ',', (@columns, 'TOTAL', 'TP', 'FP', 'FN', 'ACC');
     $n = 1;
     for my $expected (@expected) {
-        $stats{$expected}{'fp'} //= 0;
         my $lastIndex = 0;
+        my $index;
         for my $predicted (sort keys %{$matrix->{$expected}}) {
+            # Calculate the index of the label in the array of columns
             $index = _findIndex($predicted, \@columns);
+            # Print some commas to get to the column of the next value predicted
             $array[$n] .= ',' x ($index - $lastIndex) . %{$matrix}{$expected}->{$predicted};
             $lastIndex = $index;
         }
 
+        # Get to the columns of the stats
         $array[$n] .= ',' x (scalar(@columns) - $lastIndex + 1);
         $array[$n] .= join ',', (
             $stats{$expected}{'total'},
@@ -67,6 +76,7 @@ sub makeConfusionMatrix {
         );
         ++$n;
     }
+    # Print the TOTAL row to the csv file
     $array[$n] = 'TOTAL,';
     map {$array[$n] .= $totals{$_}. ','} (sort keys %totals)[0 .. $#columns];
     $array[$n] .= join ',', ($totals{'total'}, $totals{'tp'}, $totals{'fp'}, $totals{'fn'}, $totals{'acc'})
@@ -74,7 +84,7 @@ sub makeConfusionMatrix {
 
 sub _findIndex {
     my ($string, $array) = @_;
-    for (0 .. scalar(@{$array})) {
+    for (0 .. @$array - 1) {
         return $_ + 1 if ($string eq @{$array}[$_]);
     }
 }
@@ -144,6 +154,21 @@ The output will be in CSV. Here is an example:
     2015,,,,,,,,2,2,2,7,0,100.00%
     TOTAL,1,1,1,5,5,3,5,9,30,23,7,7,84.62%
 
+Prettified:
+
+
+|       | 1997 | 1998 | 2001 | 2003 | 2005 | 2008 | 2012 | 2015 | TOTAL | TP | FP | FN | ACC     |
+|-------|------|------|------|------|------|------|------|------|-------|----|----|----|---------|
+| 1997  | 1    |      |      |      |      |      |      | 1    | 2     | 1  | 0  | 1  | 50.00%  |
+| 1998  |      | 1    |      |      |      |      |      |      | 1     | 1  | 0  | 0  | 100.00% |
+| 2001  |      |      | 1    |      |      |      |      |      | 1     | 1  | 0  | 0  | 100.00% |
+| 2003  |      |      |      | 5    |      |      |      | 2    | 7     | 5  | 0  | 2  | 71.43%  |
+| 2005  |      |      |      |      | 5    |      |      | 4    | 9     | 5  | 0  | 4  | 55.56%  |
+| 2008  |      |      |      |      |      | 3    |      |      | 3     | 3  | 0  | 0  | 100.00% |
+| 2012  |      |      |      |      |      |      | 5    |      | 5     | 5  | 0  | 0  | 100.00% |
+| 2015  |      |      |      |      |      |      |      | 2    | 2     | 2  | 7  | 0  | 100.00% |
+| TOTAL | 1    | 1    | 1    | 5    | 5    | 3    | 5    | 9    | 30    | 23 | 7  | 7  | 84.62%  |
+
 =over
 
 =item TP:
